#ifndef PW_CORE_MESH_HPP
#define PW_CORE_MESH_HPP

/*
 * Copyright (c) 1999-2019 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#include <pw/core/globals.hpp>
#include <pw/core/vector.hpp>

namespace pw {

class mesh {
public:

	typedef std::vector<int32_t> indexarray_t;
	typedef std::vector<vector3> vertex3array_t;
	typedef std::vector<vector2> vertex2array_t;

	enum topology_type {
		triangles,
		triangle_strip,
		triangle_fan,
		quads,
		lines,
		line_strip,
		points
	};

	void set_indices(const indexarray_t& v) { _indices = v; }
	void set_vertices(const vertex3array_t& v) { _vertices = v; }

	const indexarray_t& indices() const { return _indices; }
	const vertex3array_t& vertices() const { return _vertices; }

	void reset();

protected:

	indexarray_t _indices; //!< indices according to topology type
	vertex3array_t _vertices; //!< geometry data
	vertex3array_t _normals; //!< normal data
	vertex3array_t _colors; //!< color data

	std::vector<vertex2array_t> _uvs;			//!< storing multiple UV sets

	// TODO add weights, tangents etc. pp.
};

}

#endif

