#include "script_system.hpp"

#include "pw/system/window.hpp"
#include "pw/system/input.hpp"
#include "pw/system/display.hpp"

#include "pw/core/debug.hpp"

// hijacking
//#include "pw/visual/pipeline.hpp"

namespace pw {

struct window_lua : public window {
	// overridable function
	sol::function lua_update;

	window_lua(sol::this_state L)
	// default is an empty callback
		: lua_update(sol::make_reference<sol::function>(L.lua_state(), [](){}))
	{
		// set this internal callback of the window to the lua callback
		_on_update = [this](window&){this->lua_update();};
	}
};


void script_system::load(sol::table &ns)
{
	ns.new_usertype<window_lua>("window",
								sol::constructors<window_lua(sol::this_state)>(),
								"on_update",&window_lua::lua_update,
								"update",&window::update,
								"title",sol::writeonly_property(&window::set_title),
								"size",sol::property(&window::size,&window::set_size),
								"position",sol::property(&window::position,&window::set_position),
								"fullscreen",sol::property(&window::fullscreen,&window::set_fullscreen)
								);

	ns.new_usertype<input>("input",
						   "new", sol::no_constructor,
						   "get",&input::get,
						   "mouse_position",sol::readonly_property(&input::mouse_position),
						   "mouse_button",sol::readonly_property(&input::mouse_button),
						   "mouse_pressed",sol::readonly_property(&input::mouse_pressed),
						   "input_string",sol::readonly_property(&input::input_string)
						   );

	ns.new_usertype<display>("display",
							 "all",&display::all,
							 "name",sol::readonly_property(&display::name)
							 );


	//    ns.set_function("my_class_func_2", &my_class::func);


	//
	//    // hijack part
	//    //
	//	ns.new_usertype<pipeline>("pipeline",
	//                              "create",&pipeline::create
	//							  );
}

}
